#ifndef _BUFFER_IMPL_H
#define _BUFFER_IMPL_H

#include <stddef.h>
#include "Buffer.h"

class BufferImpl
{
public:
	BufferImpl(int size, Buffer*) : m_CurrentSize(0), m_SizeIncrement(size), m_Size(size), m_Buffer(NULL) { m_Buffer = new char[size]; }
	
	~BufferImpl() 
	{ delete m_Buffer; }
	
	/* GENCODE START
	typedef char* iterator;
	 GENCODE END */

	void load( char* loadData, unsigned int size )
	{
		memcpy( m_Buffer, loadData, size );
		m_CurrentSize = size;
	}

	Buffer::iterator insert( const Buffer::iterator& it, const void* data, unsigned int size )
	{
		if( sizeExceeded(it, size) ) return it;
		memcpy( m_Buffer+m_CurrentSize, data, size );
		// WATCH@ it size data
		m_CurrentSize += size;
		return m_Buffer+m_CurrentSize;
	}

	inline bool sizeExceeded( const Buffer::iterator& it, unsigned int size ) { return (it+size) > (m_Buffer+m_Size); }
	
	Buffer::iterator begin()	{ return m_Buffer;			}
	Buffer::iterator end()		{ return m_Buffer+m_CurrentSize;	}
	unsigned int size()		{ return m_CurrentSize;			}	
	char* get()			{ return m_Buffer;			}
	
	void clear()
	{
		delete m_Buffer;
		m_Buffer = NULL;
		m_CurrentSize = 0;
	}
	
	
private:
	int m_Size;
	int m_CurrentSize;
	int m_SizeIncrement;
	char* m_Buffer;
};

#endif
