/* ****************************************************************************
 *
 *  MakefileGen.cpp
 *  Created by John F. Hubbard, on Fri Aug 11 2000, 14:30:27 PST
 *
 *  Copyright (c) 2000, ATD Azad Technology Development Corporation
 *
 *            The Reliable Software Outsource Resource
 *        You hire us, we do it for you, and we do it right.
 *
 *                       www.azadtech.com
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *  File Contents: Implementation of the MakefileGen class.
 *  Please see MakefileGen.h for documentation of this class.
 *
 * %version: 3.03 %
 * %date_modified: 10 August 2001 15:59:02 2001 %
 * %created_by: jhubbard %
 *
 *************************************************************************** */
#ifdef WIN32
	#pragma warning (disable : 4786) // disable MSVC's useless template length warning
#endif // WIN32

#include <string>
#include <iostream>
#include "MakefileGen.h"

namespace code_gen
{

	using std::endl;

	MakefileGen::MakefileGen(const DigestedCommands& commands)
	throw(std::string)
	:   CodeGenerator   (commands),
	mWriter         (commands.GetMakefile(),
					 commands.GetWriteSentryMode(),
					 commands.WillCreateMakefile())
	{
	}


	MakefileGen::~MakefileGen()
	{

	}

	std::ostream&
	MakefileGen::GetOstream()
	{
		return mWriter.GetOstream();
	}

	void
	MakefileGen::GenCommentHeader()     throw(std::string)
	{
		GetOstream() <<
		"#########################################################################"		<< endl	<<
		"#  " << mCommands.GetMakefile() <<	""											<< endl	<<
		"#  Created by " <<	mCommands.GetAuthor() << " on "	<< DateStamp()				<< endl	<<
		"#"																				<< endl	<<
		"#  Copyright (C) 2001, John F. Hubbard"			<< endl	<<
		"#"																				<< endl	<<
		"#  Permission is granted to use this code without restriction,"				<< endl	<<
		"#  as long as this copyright notice appears in all source files."				<< endl	<<
		"#"																				<< endl	<<
		"#  File Contents: makefile"						 						<< endl	<<
		"#"																				<< endl	<<
		"#  $" "Revision: 1.0 $"														<< endl	<<
		"#  $" "Date: "		<< DateStamp()				<< " $ "						<< endl	<<
		"#  $" "Author: "	<< mCommands.GetAuthor()	<< " $"							<< endl	<<
		"#"																				<< endl	<<
		"#########################################################################"		<< endl	<<
		endl																			<< endl;
	}

	void
	MakefileGen::GenPreface()           throw(std::string)
	{
		GetOstream() <<
		"PROGRAM=ut" <<	mCommands.GetBaseFilename()			 											<< endl	<<
		"CPP_OBJS=            ${TARGET}/"	<< mCommands.GetBaseFilename()	<< ".o   "					<< endl	<<
		"UNIT_TEST_OBJS=      ${TARGET}/ut"	<< mCommands.GetBaseFilename()	<< ".o"						<< endl	<<
		""																								<< endl	<<
		"#TARGET=SOLARIS_SPARC"																			<< endl	<<
		"#TARGET=SOLARIS_X86"																			<< endl	<<
		"#TARGET=LINUX_X86"																				<< endl	<<
		"#TARGET=LYNXOS_X86"																			<< endl	<<
		"#TARGET=LYNXOS_PPC"																			<< endl	<<
		"#TARGET=WIN32_GCC"																				<< endl	<<
		""																								<< endl	<<
		"# Change the value of these two directories to point to a standards-compliant C++"				<< endl	<<
		"# Library.  If you are using STL Port (free, from http://www.stlport.org), then no"			<< endl	<<
		"# changes are necessary."																		<< endl	<<
		""																								<< endl	<<
		"CPP_LINK_LIBRARY=${HOME}/cvs_wa/atc/thirdparty/STLport-4.5/lib/${TARGET}/libstlport_gcc.a"		<< endl	<<
		"CPP_INCLUDE_DIRECTORY=${HOME}/cvs_wa/atc/thirdparty/STLport-4.5/stlport"						<< endl	<<
		""																								<< endl	<<
		"INCLUDES=-I${CPP_INCLUDE_DIRECTORY} -I. # -I../../ClassCreator/Code -I../../ClassCreator/Code/tools "									<< endl	<<
		"LIBS:=  ${CPP_LINK_LIBRARY}"																	<< endl	<<
		""																								<< endl	<<
		"LOCAL_CXX:=gcc"																				<< endl	<<
		"LINK:=gcc"																						<< endl	<<
		"LINK_FLAGS= -lgcc -lm "																		<< endl	<<
		"OPTIMIZE_FLAGS=           # -O2"		   														<< endl	<<
		"DEBUG_FLAGS=              -g"																	<< endl	<<
		"WARNING_LEVEL_FLAGS=      -Wall -Wno-unknown-pragmas -Wno-unused"								<< endl	<<
		""																								<< endl	<<
		"CXX_FLAGS:=-D${TARGET}                \\"														<< endl	<<
		"           -D_REENTRANT               \\"														<< endl	<<
		"           ${OPTIMIZE_FLAGS}          \\"														<< endl	<<
		"           ${DEBUG_FLAGS}             \\"														<< endl	<<
		"           ${WARNING_LEVEL_FLAGS}"																<< endl	<<
		""																								<< endl	<<
		"all: ${TARGET}/${PROGRAM} "																	<< endl	<<
		""																								<< endl	<<
		"doc:"																							<< endl	<<
		"	@if [ ! -d html ];then mkdir html;fi"														<< endl	<<
		"	-@ doc++ -d html -H -u *.h"																	<< endl	<<
		""																								<< endl	<<
		"cleandoc:"																						<< endl	<<
		"	-@if [ -d html ];then rm -r html;fi"														<< endl	<<
		""																								<< endl	<<
		"setup:"																						<< endl	<<
		"	@echo \"Building the ${PROGRAM} program...\""												<< endl	<<
		"	@if [ ! -d ${TARGET} ];then mkdir ${TARGET};fi"												<< endl	<<
		""																								<< endl	<<
		"${TARGET}/${PROGRAM}: setup ${CPP_OBJS} ${UNIT_TEST_OBJS}"										<< endl	<<
		"	${LINK} ${CPP_OBJS} ${UNIT_TEST_OBJS} ${LIBS} ${LINK_FLAGS} -o ${TARGET}/${PROGRAM}"		<< endl	<<
		""																								<< endl	<<
		"${UNIT_TEST_OBJS}: ${TARGET}/%.o: %.cpp"														<< endl	<<
		"	${LOCAL_CXX} -c ${CXX_FLAGS} $< -o $@ ${INCLUDES}"											<< endl	<<
		""																								<< endl	<<
		"${CPP_OBJS}: ${TARGET}/%.o: %.cpp %.h %.icc"													<< endl	<<
		"	${LOCAL_CXX} -c ${CXX_FLAGS} $< -o $@ ${INCLUDES}"											<< endl	<<
		""																								<< endl	<<
		"cleanall: clean cleandoc "																		<< endl	<<
		""																								<< endl	<<
		"clean:"																						<< endl	<<
		"	-@if [ -d html ];then rm html/*;fi"															<< endl	<<
		"	-@if [ -d ${TARGET} ];then rm -r ${TARGET} ;fi"												<< endl	<<
		""																								<< endl	<<
		"local_test: all"																				<< endl	<<
		"	@$(TARGET)/$(PROGRAM)"																		<< endl	<<
		""																								<< endl	<<
		"dev_test: all"																					<< endl	<<
		"	@$(TARGET)/$(PROGRAM) > $(PROGRAM).out"														<< endl	<<
		""																								<< endl	<<
		"	@if cmp -s $(PROGRAM).sig $(PROGRAM).out; then      \\"		<< endl	<<
		"		echo \"UnitTest $(PROGRAM):	SUCCESS\";          \\"		<< endl	<<
		"	else                                                \\"		<< endl	<<
		"		if [ -x $(PROGRAM).sig ]; then                  \\"		<< endl	<<
		"			echo \"UnitTest $(PROGRAM):	FAILED\";       \\"			<< endl	<<
		"			diff $(PROGRAM).sig $(PROGRAM).out;         \\"			<< endl	<<
		"		else                                            \\"		<< endl	<<
		"			echo \"UnitTest $(PROGRAM):	UNKNOWN (no signature file exists)\"; \\"				<< endl	<<
		"		fi                                               \\"		<< endl	<<
		"	fi"																							<< endl	<<
		""																								<< endl;
	}

} //end of code_gen namespace
