%{
/* ****************************************************************************
 *
 *  Calculator.l
 *  Created by John F. Hubbard, on Sun Aug 26 2001, 16:36:55
 *
 *  Copyright (C) 2001, John F. Hubbard
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *
 *  File Contents: Lexer for a simple calculator. This is intended for use
 *	by a lexer generator; specifically, GNU's (www.gnu.org) flex program.
 *
 *  $Revision: 1.2 $
 *  $Date: 2001/12/04 06:18:16 $
 *  $Author: jhubbard $
 *************************************************************************** */

#include "Calculator.tab.h"
#include <string.h> // For atof
#include "BisonWorkarounds.h"

const int C_COMMENT_TYPE 	= 100;
const int CPP_COMMENT_TYPE 	= 200;
int   fCommentType 			= 0;
int	gLineNumber 			= 1;

%}

%option noyywrap

%x COMMENT
%%
"/\*"					{ BEGIN COMMENT; fCommentType = C_COMMENT_TYPE;  }
"//"					{ BEGIN COMMENT; fCommentType = CPP_COMMENT_TYPE; }
"#"				  		{ BEGIN COMMENT; fCommentType = CPP_COMMENT_TYPE; }
<COMMENT>"*/"   		{ if ( C_COMMENT_TYPE == fCommentType)   BEGIN 0; }
<COMMENT>"\n"   		{ if ( CPP_COMMENT_TYPE == fCommentType) BEGIN 0; ++gLineNumber; }
<COMMENT>.				{ ;  }

[-+]?[0-9]+(\.[0-9]*)?	{ yylval.double_value = atof(yytext);   return NUMBER; }
[\+]  					{ yylval.operation = yytext[0];   return PLUS; }
[\-]  					{ yylval.operation = yytext[0];   return MINUS; }
[\*]  					{ yylval.operation = yytext[0];   return MULTIPLY; }
[\/]  					{ yylval.operation = yytext[0];   return DIVIDE; }
[\(]  					{ yylval.char_value = yytext[0];   return OPEN_PAREN; }
[\)]  					{ yylval.char_value = yytext[0];   return CLOSE_PAREN; }
[\;]  					{ yylval.char_value = yytext[0];   return SEMICOLON; }
[ \t] 			;
[\n]					{ ++gLineNumber; }
%%

