%{
/* ****************************************************************************
 *
 *  Calculator.y
 *  Created by John F. Hubbard, on Sun Aug 26 2001, 16:36:55
 *
 *  Copyright (C) 2001, John F. Hubbard
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *  File Contents: Grammar for a simple calculator. This is intended for use
 *	by a parser generator; specifically, GNU's (www.gnu.org) bison program.
 *
 *  $Revision: 1.3 $
 *  $Date: 2001/12/05 03:18:32 $
 *  $Author: jhubbard $
 *************************************************************************** */
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include <stdlib.h>
#include <string>
#include <iostream>
#include "CommandLine.h"
#include "BisonWorkarounds.h"

#define YYERROR_VERBOSE 1
#define YYDEBUG 1
#define YYPARSE_PARAM apCommands

using namespace tools;

// apCommands is the parameter that is passed into yyparse():
#define USE_COMMANDS() 				\
	assert(apCommands); 				\
	CommandLine&  commands = *(reinterpret_cast< CommandLine* >(apCommands));  \

extern int gLineNumber;

%}
%union
{
	int			operation;
	char		char_value;
	double		double_value;
}
%left <double_value> NUMBER
%left <char_value> PLUS MINUS MULTIPLY DIVIDE SEMICOLON OPEN_PAREN CLOSE_PAREN
%type <double_value> definition_list result
					 addition subtraction multiplication division
%%

complete_pass: calculation_list
	{
		USE_COMMANDS();
		if( commands.Exists("trace_grammar" ))
		{
			std::cout << "Successfully parsed the entire input." << std::endl;
		}
	}
	;

calculation_list:  calculation
			|  calculation SEMICOLON
			|  calculation SEMICOLON  calculation_list
	;

calculation: definition_list
	{
		USE_COMMANDS();
		if( commands.Exists("trace_grammar" ))
		{
			std::cout << "Found a complete calculation." << std::endl;
		}
		std::cout << "Final result = " << $1 << std::endl << std::endl;
	}
	;

addition:   NUMBER PLUS result
	{
		USE_COMMANDS();
		if( commands.Exists("trace_grammar" ))
		{
			std::cout << "Found a valid calculator addition operation: " <<
					$1 << " + " << $3 << " ."<< std::endl;
		}
		$$ = $1 + $3;
		if( commands.Exists("show_intermediate_results" ))
		{
			std::cout << "Intermediate result = " << $$ << std::endl;
		}
	}
	;

subtraction:   NUMBER MINUS result
	{
		USE_COMMANDS();
		if( commands.Exists("trace_grammar" ))
		{
			std::cout << "Found a valid calculator subtraction operation: " <<
					$1 << " - " << $3 << " ."<< std::endl;
		}
		$$ = $1 - $3;
		if( commands.Exists("show_intermediate_results" ))
		{
			std::cout << "Intermediate result = " << $$ << std::endl;
		}
	}
	;

multiplication:   NUMBER MULTIPLY result
	{
		USE_COMMANDS();
		if( commands.Exists("trace_grammar" ))
		{
			std::cout << "Found a valid calculator multiplication operation: " <<
					$1 << " * " << $3 << " ."<< std::endl;
		}
		$$ = $1 * $3;
		if( commands.Exists("show_intermediate_results" ))
		{
			std::cout << "Intermediate result = " << $$ << std::endl;
		}
	}
	;

division:   NUMBER DIVIDE result
	{
		USE_COMMANDS();
		if( commands.Exists("trace_grammar" ))
		{
			std::cout << "Found a valid calculator division operation: " <<
					$1 << " / " << $3 << " ."<< std::endl;
		}
        if( $3 != 0.0 )
        {
            $$ = $1 / $3;
        }
        else
        {
            $$ = 0.0;
            std::cout << "You attempted to divide by zero. This won't work out too well." << std::endl;
        }
		if( commands.Exists("show_intermediate_results" ))
		{
			std::cout << "Intermediate result = " << $$ << std::endl;
		}
	}
	;

result: addition 		{ $$ = $1; }
		| subtraction  		{ $$ = $1; }
		| multiplication  	{ $$ = $1; }
		| division 			{ $$ = $1; }
		| NUMBER 			{ $$ = $1; }
		| OPEN_PAREN definition_list CLOSE_PAREN 	{ $$ = $2; }
		  ;

definition_list:  result 					{ $$ = $1; }
		| result PLUS definition_list  		{ $$ = $1 + $3; }
		| result MINUS definition_list  	{ $$ = $1 - $3; }
		| result MULTIPLY definition_list  	{ $$ = $1 * $3; }
		| result DIVIDE definition_list  	{ $$ = $1 / $3; }
	;

// End of CommonDefinitions
///////////////////////////////////////////////////////////////////////////////

%%

extern "C"
{

int
yyerror (const char* strError)
{
  std::cout << "Calculator, line " << gLineNumber << ": " << strError << std::endl;
  exit(1);
}

} // extern "C"

