/* ****************************************************************************
 *
 *  HelperRoutines.h
 *  Created by John F. Hubbard, on Sun Aug 26 2001, 19:57:53
 *
 *  Copyright (C) 2001, John F. Hubbard
 *
 *  Permission is granted to use this code without restriction,
 *  as long as this copyright notice appears in all source files.
 *
 *
 *  File Contents: Interface and documentation of the HelperRoutines component.
 *
 *  $Version: 1.0 $
 *  $Date: 2001/12/04 05:59:22 $
 *  $Author: jhubbard $
 *************************************************************************** */

#ifndef JFH_OUT_OF_BAND_EXAMPLE_HELPERROUTINES_H
#define JFH_OUT_OF_BAND_EXAMPLE_HELPERROUTINES_H

namespace tools
{
	class CommandLine; // Forward declaration
}

namespace out_of_band_example
{
	/** FeedFileToSubprogram() reads a file named aFileName, and pipes the contents
		to the standard input (stdin) of the child program that is specified
		by aSubprogramPath.
	*/
	void FeedFileToSubprogram( const std::string& aFileName,
							   const std::string& aSubprogramPath,
							   const tools::CommandLine& commands );
	
	/** GetEnv() provides a C++ style version of the Unix function getenv().
		Please type "man getenv" at the Unix command line for more information.
		This routine returns an empty std::string object upon failure to
		retrieve the environmental variable specified by variableName.
	*/
	std::string GetEnv(const std::string& variableName);

	/** CombinePathAndFile() combines a path and a file name, and correctly handles
		the intervening, single forward slash.
	*/
	std::string CombinePathAndFile( const std::string& aPath, const std::string& aFileName);

} // end of the out_of_band_example namespace

#endif // #ifndef JFH_OUT_OF_BAND_EXAMPLE_HELPERROUTINES_H

