#!/usr/bin/env ruby

require 'classdesc'

class Aspects
	private_class_method :new
end

class Errors < Aspects
	@@instance = nil

	def Errors.create
		@@instance = new unless @@instance
		@@instance
	end
	
	def initialize
		@errorBlocks = []
		File.open("errors", "r") { |file| file.read }.split("};").each { |block|
			block.strip!
			if block != ""
				createBlocks(block)
			end
		}
	end

	def createBlocks(block)
		sblock = block.split("\n")
		meth = MyMethod.new(sblock[0],"")
		i=1; excblocks = []
		while i < sblock.size
			exc = sblock[i]; i += 1; excblock = ""
			while i < sblock.size 
				excblock += sblock[i] + "\n"
				i += 1
				break if sblock[i-1] == "}"
			end
			excblocks << [ exc, excblock ]
		end
		@errorBlocks << [ meth, excblocks ]
	end
	
	def find(meth)
		@errorBlocks.find { |block| block[0].eql?(meth) }
	end
end

class Contracts < Aspects
	@@instance = nil

	def Contracts.create
		@@instance = new unless @@instance
		@@instance
	end

	def initialize
		@classInvarient, @methPreConditions, @methPostConditions = [], [], []
		File.open("contracts", "r") { |file| file.read }.split("};").each { |block|
			case block
			when /INV/
				@classInvarient << block
			when /MPRE/
				@methPreConditions << [ MyMethod.new(block.strip.split('\n')[0],""), block ]
			when /MPOST/
				@methPostConditions << [ MyMethod.new(block.strip.split('\n')[0],""), block ]
			end
		}
	end

	def findClassInvarient(className)
		c=nil
		return "#{c.gsub(/.*INV:\s*#{className}/, '').strip}\n}" if c=@classInvarient.find { |item| item =~ /:\s+(#{className}[\w\d_]*)/; $1 == className }
		c
	end

	def findMethodPreCondition(meth)
		c=nil
		c = @methPreConditions.find { |item| item[0].eql?(meth) }
		return "// pre conditions block\n#{c[1].gsub(/^(.*MPRE:|\{).*?\n/, '').strip}\n" if c
		c
	end

	def findMethodPostCondition(meth)
		c=nil
		c = @methPostConditions.find { |item| item[0].eql?(meth) }
		return "// post conditions block\n#{c[1].gsub(/^(.*MPOST:|\{).*?\n/, '').strip}\n" if c
		c
	end

end

class Watches < Aspects
	@@instance = nil

	def Watches.create
		@@instance = new unless @@instance
		@@instance
	end

	def initialize
		@items = []
		File.open("watches", "r") { |file|
			lines = []; svline = ""
			while file.gets
				case $_
				when /\{/
					while file.gets && $_.chomp != "};"
						l = $_.split(":")
						lines << [l[0].to_i, l[1].strip.split]
					end
					@items << [ MyMethod.new(svline), lines ]
					lines = []
				end
				svline = $_
			end
		}
	end
	
	def find(meth)
		return true if @items.find { |item| item[0].eql?(meth) }
		false
	end
end

