#ifndef _CALLSTATEIMPL_H
#define _CALLSTATEIMPL_H

#include "SimpleCall.h"
#include "CallState.h"
#include "LogRecord.h"
#include "LogRecordSpec.h"
#include "LogStore.h"

/* GENCODE START
#include <string>
class CallStateException
{
public:
	CallStateException( const string& msg ) : m_msg(msg)
	{
	}
	const string& getMsg() { return m_msg; }
private:
	string m_msg;
};
 * GENCODE END */

class CallStateImpl
{
public:
	CallStateImpl( SimpleCall* callobj ) : m_callobj(callobj)
	{
	}
	virtual ~CallStateImpl()
	{
	}

	virtual const string& getStringStateName() 
	{
		return m_name;
	}

	// Define State Machine Inputs
	virtual void inputMakeCall()
	{
		// invalid input for this state - indicates that a call should be started
		throw InvalidStateException(getStringStateName());
	}
	virtual void inputRouteComplete()
	{
		// invalid input for this state - indication from the network that the call is routed
		throw InvalidStateException(getStringStateName());
	}
	virtual void inputAnswer()
	{
		// invalid input for this state - indicates called party has answered
		throw InvalidStateException(getStringStateName());
	}
	virtual void inputDisconnectCalled()
	{
		// invalid input for this state - indicates that called party hung up
		throw InvalidStateException(getStringStateName());
	}
	virtual void inputDisconnectCalling()
	{
		// invalid input for this state - indicates that calling party hung up
		throw InvalidStateException(getStringStateName());
	}
	virtual void inputBillCall()
	{
		// invalid input for this state - indicates that billing should be completed for the call
		throw InvalidStateException(getStringStateName());
	}

protected:
	SimpleCall*	m_callobj;
	string		m_name;
};

#endif
