#require 'aspectref'

class Param
	# ex., type="int", qualifier="const", refPtr="&", name="x", initializer="1024" (i.e., x=1024)
	attr_accessor :unsigned, :type, :const, :amp, :ptr, :ptrptr, :name, :initializer

	def initialize( param )
		@type, @unsigned, @const, @amp, @ptr, @ptrptr, @name, @initializer = "", false, false, false, false, false, "", ""
		parseParam( param )
	end

	def parseParam( param )
		@const = true if param.include?("const")
		@amp = true if param.include?("&")
		@unsigned = true if param.include?("unsigned")
		if param.include?("**") then @ptrptr = true else @ptr = true if param.include?("*") end
		param[/=\s*(\"*\s*[\w\d]+\s*\"*)/]; @initializer = $1
		param.gsub!(/const|\*|&|unsigned/, ''); param.gsub!(/=.*$/,''); param.strip!
		pa = param.split()
		@name = pa[1]
		@type = pa[0]
	end

	def toStringFull
		"#{\"const \" if @const}#{\"unsigned \" if @unsigned}#{@type}#{\"*\" if @ptr}#{\"**\" if @ptrptr}#{\"&\" if @amp}  #{@name}#{\"=\" if @initializer}#{@initializer if @initializer}"
	end

	def toStringNoInit
		"#{\"const \" if @const}#{\"unsigned \" if @unsigned}#{@type}#{\"*\" if @ptr}#{\"**\" if @ptrptr}#{\"&\" if @amp}  #{@name}"
	end

	def toStringNames
		@name
	end
	
	def eql?( param )
		return type == param.type && const == param.const && amp == param.amp && ptr == param.ptr && ptrptr == param.ptrptr && unsigned == param.unsigned
	end
end

class Params
	attr_accessor :params
	def initialize( params )
		@params = []
		parseParams(params)
	end

	def parseParams(params)
		return if params.nil?
		params.gsub!(/[()\r]/, '')
		return if params.sub(/\s*/,'') == ""
		params.split(',').each { |item| @params << Param.new(item) }
	end

	def size
		@params.size
	end
	
	def toString()
		str = ""; pr=" "
		@params.each { |param| str += pr + yield(param); pr=", " }
		str
	end

	def toStringFull
		toString { |param| param.toStringFull }
	end

	def toStringNoInit
		toString { |param| param.toStringNoInit }
	end

	def toStringNames
		toString { |param| param.toStringNames }
	end
	
	def eql?( params )
		@params.eql?(params.params)
	end
end

class MyMethod 
	attr_accessor :params, :inline, :const, :amp, :ptr, :ptrptr, :retType, :name, :constructor

	def initialize(meth, constructor)
		@inline, @unsigned, @const, @static, @amp, @ptr, @ptrptr, @retType, @name, @constructor = false, false, false, false, false, false, false, "", "", constructor
		parseMethod meth
	end

	def eql?(meth)
		return meth.name == @name && params.eql?(meth.params) if @name.count("?*[\|") == 0
		meth.name =~ Regexp.new(@name) && params.eql?(meth.params)
	end
	
	def parseMethod(m)
		meth = m.dup
		meth[/^\s*[\w\d:_]+.*?(\(.*?\))/]
		@params = Params.new( $1 )
		meth.gsub!(/\(.*\).*$/, '')
		@inline = true if meth.include?("inline")
		@const = true if meth.include?("const")
		@amp = true if meth.include?("&")
		@unsigned = true if meth.include?("unsigned")
		@static = true if meth.include?("static")
		if meth.include?("**") then @ptrptr = true else @ptr = true if meth.include?("*") end
		meth.gsub!(/virtual|inline|const|static|unsigned|\*|&/, ''); meth.strip!; 
		ma=meth.split
		if( ma.size == 2 ) then @name = ma[1]; @retType = ma[0] else @name = ma[0] end
	end
	
	def toStringDeclare
		str = "	#{\"inline \" if @inline}#{\"static \" if @static}#{\"const \" if @const}#{\"unsigned \" if @unsigned}#{@retType}#{\"*\" if @ptr}#{\"**\" if @ptrptr}#{\"&\" if @amp} #{@name}("
		str += @params.toStringFull 
		str += ");"
	end

	def toStringDef(className,abstractFlag,&block)
		str = "#{\"const \" if @const}#{\"unsigned \" if @unsigned}#{@retType}#{\"*\" if @ptr}#{\"**\" if @ptrptr}#{\"&\" if @amp} #{className}::#{@name}("
		str += @params.toStringNoInit 
		if @constructor then str += ") { m_impl = new #{@constructor}("
		elsif @static then str += ") { return #{className}Impl::#{@name}("
		else str += ") { #{\"return \" if @retType != \"void\"}m_impl#{className if abstractFlag}->#{@name}("
		end
		str += @params.toStringNames 
		str += "#{\", \" if @params.size > 0} this" if @constructor
		str += "); #{block.call(\"\") if @constructor} }"
	end
end

class ClassDesc
	attr_accessor :methods, :name, :haveConstructor, :precondition, :postcondition

	def initialize(baseName,name)
		@methods, @baseName, @name, @inClassCode, @extClassCode, @inheritance = [], baseName, name, [], [], []
	end

	def addConstructor(meth,constName)
		@methods << MyMethod.new(meth,constName)
	end

	def addInheritance(str)
		str.scan(/public\s+.*?Impl/).each { |item| @inheritance << item.sub(/public\s+/,'').sub('Impl','') }
	end
	
	def addMethod(meth)
		@methods << MyMethod.new(meth,nil)
	end

	def lastMethod
		@methods.last
	end

	def addInClassCode(code)
		@inClassCode << code
	end

	def addExtClassCode(code)
		@extClassCode = code
	end

	def getPreConditionCode
		return "void _classPreCondition()\n#{@precondition}" if @precondition
	end

	def getPostConditionCode
		return "void _classPostCondition()\n#{@postcondition}" if @postcondition
	end

	def outputInhList
		str = " :"; pre = ""
		@inheritance.each { |item| str += pre + " public " + item; pre="," }
		str
	end
	
	def outputInhInit(tag)
		str = ""
		@inheritance.each { |item| str += item + "::setImpl(m_impl#{tag}); " }
		str
	end
	
	def createHdrFile(abstractFlag)
		File.open("#{@name}Impl.hi", "w+") { |file|
			file.puts "#ifndef _#{@name}_H"
			file.puts "#define _#{@name}_H\n"
			@inheritance.each { |item| file.puts "#include \"#{item}.h\"" }
			@extClassCode.each { |str| file.puts str }
			file.puts "class #{@baseName}Impl;\n"
			file.puts "class #{@name} #{outputInhList if @inheritance.size > 0}\n{"
			file.puts "public:"
			file.puts "	~#{@name}();" unless abstractFlag
			@inClassCode.each { |str| file.puts str }
			@methods.each { |meth| file.puts meth.toStringDeclare }
			file.puts "	void setImpl(#{@baseName}Impl* impl);" if abstractFlag
			file.puts "private:"
			file.puts "	#{@baseName}Impl* m_impl#{@baseName if abstractFlag};"
			file.puts "};\n"
			file.puts "#endif"
		}
	end

	def createCFile(abstractFlag)
		File.open("#{@name}Impl.ci", "w+") { |file|
			file.puts "#include \"#{@baseName}Impl.h\"\n"
			file.puts  "#{@baseName}::~#{@name}() { delete m_impl; }" unless abstractFlag
			@methods.each { |meth| file.puts meth.toStringDef(@name,abstractFlag,&method("outputInhInit")) }
			file.puts "void #{@baseName}::setImpl(#{@baseName}Impl* impl) { m_impl#{@baseName} = impl; #{outputInhInit(@baseName)} }" if abstractFlag
		}
	end
end

class Interfaces
	attr_accessor :className
	def initialize(className)
		@interfaces, @className = [], className
	end
	def addClass(c)
		@interfaces << c
	end
	def createFiles(abstractFlag)
		@interfaces.each { |intf| intf.createHdrFile(abstractFlag); intf.createCFile(abstractFlag) }
	end
	def lastClass
		@interfaces.last
	end
	def addExtClassCode(code)
		@interfaces.each { |intf| intf.addExtClassCode(code) } if code.size > 0
	end
end

