#!/usr/bin/env ruby

require 'logstring'
require 'logspec'
require 'aspects'

$line = ""
$file = ARGV[0]
$lastStatement = ""
$inmethod = false
$privilege = ""
$currClass = nil
$watchindex = 1
$errorindex = 1
$funcname = ""
$fullFunctionDef = ""
$currMethod = nil
$errorBlock = nil
$watchexp = Regexp.new(/(.*?)\/\/\s*WATCH@\s*(.*)/)
$errorexp = Regexp.new(/(.*?)\/\/\s*ERROR@\s*(.*?)\s+(.*)/)
$logspec = LogSpec.create

class String
	def addTabs(tabs)
		newstr = ""
		each { |item| newstr += "\t"*tabs + item }
		newstr
	end
end

def findContract( contractMethod, contractArg, preString, tabs )
	return preString + "\n"  unless bl = Contracts.create.send(contractMethod, contractArg)
	preString + "\n" + processBlock(bl).addTabs(tabs) + "\n"
end

def genClassContracts( className, tabs )
	invblock = Contracts.create.findClassInvarient(className)
	invblock = "{\n}" unless invblock
	"	void invarient()\n" + processBlock(invblock).addTabs(tabs)
end

def processMethod( funcName, fullFuncDef, constructor )
	$funcname = funcName
	$fullFunctionDef = fullFuncDef.sub(/(\s*\w[\w\d_]+\s*\()/, $currClass + '::\1')
	$currMethod = MyMethod.new($fullFunctionDef,"")
	$watchindex = 1; $errorindex = 1
	tabs = $line.count("\t")+1
	constructor ? preString="" : preString = "invarient();".addTabs(tabs)
	$line = $line + findContract(:findMethodPreCondition, $currMethod, gets + preString, tabs ) if $privilege == "public" 
	$inmethod = true
	$errorBlock = Errors.create.find($currMethod)
	$line += "try {".addTabs(tabs) if $errorBlock
end

def processBlock(block)
	retstr = ""
	windex = 1
	errindex = 1
	block.each { |line| 
		if line =~ $watchexp then retstr += genLogRecord("INFO", "WATCH", $2, windex ); windex += 1
		elsif line =~ $errorexp then retstr += genLogRecord($2, "ERROR", $3, errindex ); errindex += 1 
		else retstr += line
		end
	}
	retstr
end

def genErrorBlocks(tabs)
	return unless $errorBlock
	errstr = "}\n".addTabs(tabs)
	$errorBlock[1].each { |exceptionEntry| errstr += "catch(#{exceptionEntry[0]}& e)\n#{processBlock(exceptionEntry[1])}".addTabs(tabs) }
	$line = errstr + "\n" + $line
end

def genLogRecord(criticality, type, vars, index)
	lrecvar = "lrec" + type + index.to_s
	str = "LogRecord #{lrecvar}(#{criticality}, #{insertString(\"simple\")}, #{insertString(type)}, #{insertString($currClass + \"::\" + $funcname + \" \" + type + \" \" + index.to_s)}, #{insertString(Dir.getwd + \"/\" + $file)}, __LINE__);"
	vars.split(/\s+/).each { |var|
		str += " lrecInsert( #{lrecvar}, #{insertString(var)}, #{var} );"
	}
	str += " LogStore::GetInstance()->store(&#{lrecvar});\n"
#	str += " #{lrecvar}.print( cout );\n"
end

def getLogInsertString( className, tabs )
	return  "" unless (line=$logspec.find(className))
	line[/\w[\w\d_]*\s+\w[\w\d_\.]*\s+(.*)/]; vars = $1.split
	str = "\npublic:\n"
	str += "void lrecInsertEx(LogRecord& lrec, unsigned int name )\n{\n".addTabs(tabs)
	str += "lrec.insert(name);\nlrec.insert(LogNV);\nlrec.insert(\"#{className}\");\nlrec.insert(#{vars.size});\n".addTabs(tabs+1)
	vars.each { |var| str += "lrecInsert(lrec, #{insertString(var)}, #{var} );\n".addTabs(tabs+1) }
	str += "}\n".addTabs(tabs)
	str += "private:\n"
end

callcc{|$cc|}
while gets 
	$line = $_
	case $_

	when /(public|protected|private)\s*:/
		$privilege = $1

	#
	# identify the current class we will be working on
	#
	when /class.*?;/
	when /^\s*class\s*(\w[\w\d_]*)/
		$currClass = $1
		tabs = $line.count("\t") + 1
		$line += gets + genClassContracts( $currClass, tabs)
		$line += getLogInsertString( $currClass, tabs )

	#
	# identify the end of the current method
	#
	when /^\s{0,1}}/
		if !($lastStatement =~ /return/) && $inmethod
			tabs = $line.count("\t")+1
			$line = findContract(:findMethodPostCondition, $currMethod, "invarient();".addTabs(tabs), tabs ) + $line
		end
		genErrorBlocks( $_.count("\t")+1 ) if $inmethod
		$inmethod = false

	#
	# identify 'return' statements - we need to add post conditions at
	# each return point
	#
	when /^\s{2,}return.*[;]\s*/
		tabs = $line.count("\t")
		$line = findContract(:findMethodPostCondition, $currMethod, "invarient();".addTabs(tabs), tabs ) + $line

	#
	# identify and process watches
	#
	when $watchexp
		print $1 
		puts genLogRecord( "INFO", "WATCH", $2, $watchindex )
		$watchindex += 1
		$cc.call
		
	#
	# identify and process error aspects
	#
	when $errorexp
		print $1 
		puts genLogRecord( $2, "ERROR", $3, $errorindex )
		$errorindex += 1
		$cc.call
		
	#
	# identify the constructor(s)/destructor
	#
	when /;/
	when /^\s*(virtual){0,1}\s*(~{0,1}#{$currClass}\s*\(.*?\))/
		processMethod( $currClass, $2, true )

	#
	# identify methods in class
	#
	when /^\s+(unsigned|const|inline|virtual){0,1}\s*(const){0,1}\s*\w[\w\d&]+\s+((\w[\w\d]*)\s*\(.*?\))/
		processMethod( $4, $3, false ) 
	when /^\s*(unsigned|const|inline|virtual){0,1}\s*(unsigned|const){0,1}\s*\w[\w\d&]*\s*(\w[\w\d_]*)\s*::\s*((\w[\w\d]*)\s*\(.*?\))/
		$privilege = "public"
		$currClass = $3
		processMethod( $5, $4, false )
	end
	
	
	puts $line
	$lastStatement = $line unless $line.strip == ""
end

