#!/usr/bin/env ruby

require 'logstring'
require 'logspec'

logspec = LogSpec.create
logspec.parseSpec

fileH = File.open("LogRecordSpec.h", "w+")
fileC = File.open("LogRecordSpec.c", "w+")

fileC.puts "#include <string>", "#include <iostream.h>", "#include \"LogRecordSpec.h\""
logspec.hdrFiles.uniq.each { |hdr|
	fileC.puts "#include \"#{hdr}\"" 
}
fileC.puts "char* logStrings[] = {"
File.open("logstrings", "r") { |file|
	while file.gets	
		fileC.puts "	\"#{$_.chomp}\","
	end
}
fileC.puts "};"

fileC.puts <<-ST
void lrecInsert( LogRecord& lrec, unsigned int name, const string& value )
{
	lrec.insert( name );
	lrec.insert( LogString );
	lrec.insert( value.size()+1 );
	lrec.insert( value.c_str(), value.size()+1 );
}

void lrecInsert( LogRecord& lrec, unsigned int name, const int& value )
{
	lrec.insert( name );
	lrec.insert( LogInt );
	lrec.insert( sizeof(int) );
	lrec.insert( &value, sizeof(int) );
}

void lrecInsert( LogRecord& lrec, unsigned int name, const unsigned long& value )
{
	lrec.insert( name );
	lrec.insert( LogLong );
	lrec.insert( sizeof(value) );
	lrec.insert( &value, sizeof(value) );
}

void lrecInsert( LogRecord& lrec, unsigned int name, const unsigned int& value )
{
	lrec.insert( name );
	lrec.insert( LogInt );
	lrec.insert( sizeof(int) );
	lrec.insert( &value, sizeof(unsigned int) );
}
ST

fileH.puts <<-ST
#ifndef _LOGRECORDSPEC_H
#define _LOGRECORDSPEC_H

#include \"LogRecord.h\"

ST

logspec.classAndVars.each { |cvpair|
	fileH.puts "class #{cvpair[0].sub('Impl','')};"
}

fileH.puts <<-ST 
template<class TYPE>
inline void lrecInsert( LogRecord& lrec, unsigned int name, const TYPE& value )
{
	lrec.insert( name, &const_cast<TYPE&>(value), sizeof(TYPE) );
}

extern void lrecInsert( LogRecord& lrec, unsigned int name, const string& value );

extern void lrecInsert( LogRecord& lrec, unsigned int name, const int& value );

extern void lrecInsert( LogRecord& lrec, unsigned int name, const unsigned int& value );
ST

logspec.classAndVars.each { |cvpair| 
	vars = cvpair[1].split
	if cvpair[0] =~ /Impl/ 
		val = cvpair[0].sub('Impl','')
		fileH.puts "extern void lrecInsert( LogRecord&, unsigned int, #{val}& );"
		fileC.puts "void lrecInsert( LogRecord& lrec, unsigned int name, #{val}& value )\n{ value.lrecInsertEx(lrec, name); }"
	else 
		fileH.puts "extern void lrecInsert( LogRecord&, unsigned int, #{cvpair[0]}& );" 
		fileC.puts "void lrecInsert( LogRecord& lrec, unsigned int name, #{cvpair[0]}& value )\n{"
		fileC.puts "	lrec.insert( name );"
		fileC.puts "	lrec.insert( LogNV );"
		fileC.puts "	lrec.insert( \"#{cvpair[0]}\" );"
		fileC.puts "	lrec.insert( #{vars.size} );"
		vars.each { |var|
			fileC.puts "	lrecInsert( lrec, #{insertString(var)}, value.#{var} );"
		}
		fileC.puts "}"
	end
}

fileH.puts "#endif"
