#include <stdio.h>
#include <string>
#include <vector>
#include <list>
#include <time.h>
#include <errno.h>
#include "LogRecord.h"

enum LogArgTags
{
	AT_ALLDATA,
	AT_ENDTIME,
	AT_STARTTIME,
	AT_RECS,
	AT_LPAREN,
	AT_RPAREN,
	AT_AND,
	AT_OR,
	AT_CRIT,
	AT_TAG,
	AT_PROCESS,
	AT_TAIL,
	AT_NUMARGS
};

template <class ARGTAG>
class Params 
{
public:
	ARGTAG	m_Tag;
	string	m_Arg;
	bool	m_bData;
};


template <class ARGTAG>
class ParamsInt : public Params<ARGTAG>
{
public:
	ParamsInt<ARGTAG>() {}
	ParamsInt<ARGTAG>(Params<ARGTAG>& from) { copy(from); }
	ParamsInt<ARGTAG>& operator=(Params<ARGTAG>& from) {  copy( from ); return *this; }
	void copy( Params<ARGTAG>& from) { m_Tag = from.m_Tag; m_Arg = from.m_Arg; m_bData = from.m_bData; }
	string	m_sData;
};

template <class ARGTAG>
class Arg
{
public:
	Arg(char* argv[], int argc, Params<ARGTAG> params[], unsigned int size);

	int isset(ARGTAG);
	int getTag(ARGTAG);
	int maxArg();
	ARGTAG	getArgTag(int);
	string	getArgLabel(int);
	string	getArg(int);
	string	getArgData(ARGTAG);

protected:
	Params<ARGTAG>*	FindArg(const string& arg);
	bool GetData(string& arg, ParamsInt<ARGTAG>* pi );
	bool GetDataDirect(const string& arg, ParamsInt<ARGTAG>* pi );
	vector<ParamsInt<ARGTAG> > m_ParamsFound;
	vector<Params<ARGTAG> > m_ParamsSpec;
};

template <class ARGTAG>
Params<ARGTAG>* Arg<ARGTAG>::FindArg(const string& arg)
{
	for( int i=0; i < m_ParamsSpec.size(); i++ )
	{
		if( arg == m_ParamsSpec[i].m_Arg ) return &m_ParamsSpec[i];
	}

	return NULL;
}

template <class ARGTAG>
Arg<ARGTAG>::Arg(char* argv[], int argc, Params<ARGTAG> params[], unsigned int size)
{
	Params<ARGTAG> p;
	for( int i=0; i < size; i++ )
	{
		p = params[i];
		m_ParamsSpec.push_back(p);
	}

	ParamsInt<ARGTAG> pi;
	string arg;
	for( int i=1; i < argc; i++ )
	{
		arg=argv[i];
		Params<ARGTAG>* pParams = FindArg(arg);
		if( pParams != NULL )
		{
			pi = *pParams;
			if( pParams->m_bData )
			{
				if( !GetData(arg, &pi) )
				{
					if( (i+1) < argc && GetDataDirect((string)argv[i+1], &pi) ) i++;
					else throw (string)"syntax error: " + arg + " expected data";
				}
			}
			else if( GetData(arg, &pi) ) throw (string)"syntax error: " + arg + " did not expect data";
		}
		else throw (string)"syntax error: " + arg + " invalid argument";

		m_ParamsFound.push_back(pi);
	}
}

template <class ARGTAG>
int Arg<ARGTAG>::isset(ARGTAG tag)
{
	vector<ParamsInt<ARGTAG> >::iterator it = m_ParamsFound.begin();
	for(; it != m_ParamsFound.end(); it++ )
	{
		if( (*it).m_Tag == tag ) return 1;
	}

	return 0;
}

template <class ARGTAG>
int Arg<ARGTAG>::getTag(ARGTAG tag)
{
	for(int i=0; i < m_ParamsFound.size(); i++ )
	{
		if( m_ParamsFound[i].m_Tag == tag ) return i;
	}
	return -1;
}

template <class ARGTAG>
int Arg<ARGTAG>::maxArg()
{
	return m_ParamsFound.size();
}

template <class ARGTAG>
ARGTAG Arg<ARGTAG>::getArgTag(int index)
{
	if( index < m_ParamsFound.size() ) return m_ParamsFound[index].m_Tag;
	else return (ARGTAG)0;
}

template <class ARGTAG>
string Arg<ARGTAG>::getArgLabel(int index)
{
	if( index < m_ParamsFound.size() ) return m_ParamsFound[index].m_Arg;
	else return "";
}

template <class ARGTAG>
string Arg<ARGTAG>::getArg(int index)
{
	if( index < m_ParamsFound.size() ) return m_ParamsFound[index].m_sData;
	else return "";
}

template <class ARGTAG>
string Arg<ARGTAG>::getArgData(ARGTAG tag)
{
	for( int i=0; i < m_ParamsFound.size(); i++ )
	{
		if( m_ParamsFound[i].m_Tag == tag ) return m_ParamsFound[i].m_sData;
	}

	return "";
}

template <class ARGTAG>
bool Arg<ARGTAG>::GetData(string& arg, ParamsInt<ARGTAG>* pi)
{
	if( pi->m_Arg.length() != arg.length() )
	{
		return true;
	}
	return false;
}

template <class ARGTAG>
bool Arg<ARGTAG>::GetDataDirect(const string& arg, ParamsInt<ARGTAG>* pi)
{
	if( FindArg(arg) == NULL )
	{
		pi->m_sData = arg;
		return true;
	}
	return false;
}

typedef Arg<LogArgTags> LogArg;

class MetaFilter {
public:
	virtual int operator&(LogRecord&) = 0;

	MetaFilter*	m_left;
	MetaFilter*	m_right;
};

class MetaFilterAnd : public MetaFilter {
public:
	int operator&(LogRecord& h) 
	{
		return (*m_left & h) && (*m_right & h);
	}
};

class MetaFilterOr : public MetaFilter {
public:
	int operator&(LogRecord& h)
	{
		return (*m_left & h) || (*m_right & h);
	}
};

class MetaFilterDummy : public MetaFilter {
public:
	int operator&(LogRecord& h) { return 1; }
};

struct LogFilterOptions {
	string	m_Criticality;
	string	m_Tag;
	string	m_Component;
	string	m_Process;
};

class LogFilter : public MetaFilter {
public:
	int		operator&(LogRecord&) { /* implementation not shown */ return 0; }
	LogFilter&	operator|=(LogFilterOptions&) { /* implementation not shown */ return *this; }
private:
	list<Criticality> m_Criticality;
	list<Criticality> m_Ncriticality;
	list<long> m_Process;
	list<long> m_Nprocess;
	list<string> m_Tag;
	list<string> m_Ntag;
};

int checkArgs(LogArg* aptr)
{
	if( (aptr->isset(AT_ALLDATA) + (aptr->isset(AT_STARTTIME) || aptr->isset(AT_ENDTIME)) + aptr->isset(AT_RECS) ) > 1 )
	{
		cerr << "Must specify one of: -A, -s/-e, or -r" << endl
		     << "Cannot specify more than one of these options" 
		     << endl;
		return 0;
	}
	return 1;
}

int getFilterStart(LogArg* aptr)
{
	unsigned int start = (unsigned int)aptr->getTag(AT_LPAREN);
	if( aptr->getTag(AT_TAG) < start ) start = (unsigned int)aptr->getTag(AT_TAG);
	if( aptr->getTag(AT_PROCESS) < start ) start = (unsigned int)aptr->getTag(AT_PROCESS);
	if( aptr->getTag(AT_CRIT) < start ) start = (unsigned int)aptr->getTag(AT_CRIT);
	return start;
}

void syntaxError(LogArg* aptr, int errInd)
{
	cerr << "syntax error: ";
	int i;
	for( errInd--,i=getFilterStart(aptr); i < errInd; i++ )
	{
		cerr << aptr->getArgLabel(i) << aptr->getArg(i) << " ";
	}
	cerr << ">>> " << aptr->getArg(i) << " <<<" << endl;
}

int getFilterValues( LogArg* aptr, LogFilter& filter, unsigned int& i )
{
	LogFilterOptions opt;
	int start, done;
	for(start=i,done=0; !done && i < aptr->maxArg();)
	{
		switch( aptr->getArgTag(i) ) {
		case AT_CRIT:
			opt.m_Criticality = aptr->getArg(i);
			break;
			
		case AT_TAG:
			opt.m_Tag = aptr->getArg(i);
			break;
					
		case AT_PROCESS:
			opt.m_Process = aptr->getArg(i);
			break;

		default:
			done=1;
			continue;
		}
		i++;
	}
	filter |= opt;
	return (start!=i);
}

MetaFilter* getMF(LogArg* aptr, unsigned int& i)
{
	if( i >= aptr->maxArg() ) 
	{
		syntaxError(aptr, i);
		return NULL;
	}
	
	switch( aptr->getArgTag(i) ) {
	case AT_LPAREN:
	{
		/* Go get the left node */
		MetaFilter* left = getMF(aptr, ++i);
		if( left == NULL ) return NULL;

		else if( aptr->getArgTag(i++)  != AT_RPAREN )
		{
			syntaxError(aptr, i);
			return NULL;
		}
		else if( i >= aptr->maxArg() ) return left;
		
		/* Determine if we need to create a parent node */
		LogArgTags type = aptr->getArgTag(i);
		switch(type) {
		case AT_AND:
		case AT_OR:
		{
			/* Need to get the right node */
			MetaFilter* right = getMF(aptr, ++i);
			if( right == NULL ) return NULL;
			
			MetaFilter* parent;
			if( type == AT_AND )
			{
				parent = new MetaFilterAnd;
			}
			else parent = new MetaFilterOr;
			
			parent->m_left = left;
			parent->m_right = right;
			return parent;
		}
		default:
			break;
		}
		
		return left;
	}
	case AT_CRIT:
	case AT_TAG:
	case AT_PROCESS:
	{
		/* Must be a value set, read in the values */
		LogFilter* leaf = new LogFilter;
		if( leaf == NULL ) 
		{
			cerr << "Failed to create LogFilter object, errno "
			     << errno << endl;
			break;
		}
		int sv = i;
		if( !getFilterValues(aptr,*leaf,i) )
		{
			syntaxError(aptr,sv);
			cerr << "Expected a set of filter options" << endl;
			delete leaf;
			break;
		}
		
		if( i >= aptr->maxArg() ) return leaf;
		
		/* Now look for a relation */
		LogArgTags type = aptr->getArgTag(i);
		switch( type ) {
		case AT_AND:
		case AT_OR:
		{
			/* We need to explore the right side */
			MetaFilter* right = getMF(aptr,++i);
			if( right == NULL ) return NULL;
			
			MetaFilter* parent;
			if( type == AT_AND )
			{
				parent = new MetaFilterAnd;
			}
			else parent = new MetaFilterOr;
			
			parent->m_left = leaf;
			parent->m_right = right;
			return parent;
		}
		default:
			return leaf;
		}
	}
	default:
		syntaxError(aptr,i);
		break;
	}
	
	return NULL;
}

int convertTime( const string& str, time_t& uTime )
{
	tm ts;
	
	if( sscanf(str.c_str(),"%d/%d/%d %d:%d:%d", 
	       &ts.tm_mon, &ts.tm_mday, &ts.tm_year, &ts.tm_hour, &ts.tm_min, &ts.tm_sec ) == 6)
	{
		if( ts.tm_year > 99 ) ts.tm_year -= 1900;
		ts.tm_mon--;
		ts.tm_isdst = -1;
		uTime = mktime(&ts);
	}
	else if( sscanf(str.c_str(),"%d:%d:%d",
	         &ts.tm_hour, &ts.tm_min, &ts.tm_sec ) == 3 )
	{
		if( ts.tm_hour < 0 || ts.tm_min < 0 || ts.tm_sec < 0 )
		{
			uTime = time(NULL) + (ts.tm_hour*3600) + (ts.tm_min*60) + ts.tm_sec;
		}
		else 
		{
			time_t now = time(NULL);
			tm* ltime = localtime(&now);
			ts.tm_mon = ltime->tm_mon;
			ts.tm_mday = ltime->tm_mday;
			ts.tm_year = ltime->tm_year;
			ts.tm_isdst = -1;
			uTime = mktime(&ts);
		}
	}
	else if( sscanf(str.c_str(),"%d/%d/%d", 
	       &ts.tm_mon, &ts.tm_mday, &ts.tm_year ) == 3)
	{
		if( ts.tm_year > 99 ) ts.tm_year -= 1900;
		ts.tm_hour=ts.tm_min=ts.tm_sec=0;
		ts.tm_mon--;
		ts.tm_isdst = -1;
		uTime = mktime(&ts);
	}
	else if( sscanf(str.c_str(),"%d", &ts.tm_hour ) == 1 )
	{
		if( ts.tm_hour >= 0 ) return 0;
		
		uTime = time(NULL) + (ts.tm_hour*3600);
	}
	else return 0;
	
	return 1;
}

int main(int argc, char* argv[])
{
	int nRetCode = 0;

	string usage("USAGE: logcat [-r <records> | -s <start time> [-e <end time>] | -A] [-f] [(] [-c <criticality>,...] [-t <tag>,...] [-p <processId>,...] [-o | -a] [...] [)]\nwhere:\n \
		-c		find these set of <criticality>\n \
		-e		end time (see -s)\n \
		-f		tail the log file to stdout\n \
		-t		find these set of <tag>\n \
		-C		find these set of <component>\n \
		-r		examine the last <records> records\n \
		-s		start time (mm/dd/yy hh:mm:ss, mm/dd/yy, hh:mm:ss, -hh:mm:ss\n \
		-a		logical AND of filter options\n \
		-o		logical OR of filter options\n \
		-p		find these set of <processIDs>\n \
		-A		examine all the log data\n");

	try
	{
		Params<LogArgTags> norm[] = {
			{ AT_ALLDATA,   "-A",		0 },
			{ AT_ENDTIME,   "-e",		1 },
			{ AT_STARTTIME, "-s",		1 },
			{ AT_RECS,      "-r",		1 },
			{ AT_LPAREN,	"(",		0 },
			{ AT_RPAREN,	")" ,		0 },
			{ AT_AND,	"-a",		0 },
			{ AT_OR,	"-o",		0 },
			{ AT_CRIT,	"-c",		1 },
			{ AT_TAG,	"-t",		1 },
			{ AT_PROCESS,	"-p",		1 },
			{ AT_TAIL,	"-f",		0 }
		};
			
		Arg<LogArgTags> args(argv,argc,norm, sizeof(norm)/sizeof(Params<LogArgTags>) );

		/* Check for valid options */
		if( !checkArgs(&args) ) 
		{
			cerr << usage << endl;
			exit(1);
		}
			
		/* Determine where the filter options start */
		unsigned int start = getFilterStart(&args);
		MetaFilter* mfp;
		if( start == (unsigned int)-1 )
		{
			mfp = new MetaFilterDummy;
			if( mfp == NULL ) 
			{
				cerr << "Failed to create a MetaFilter object, errno "
					 << errno << endl;
				exit(1);
			}
		}
		else if( (mfp=getMF(&args,start)) == NULL ) 
		{
			cerr << usage << endl;
			exit(1);
		}

		MetaFilter& mf = *mfp;
			
		time_t startTime=0, endTime=0;
		bool bTime=false;
		if( args.isset(AT_STARTTIME) && args.isset(AT_ENDTIME) )
		{
			/* Both start and end times are specified */
			bTime=true;
			if(!convertTime( args.getArgData(AT_STARTTIME), startTime) ||
					startTime > time(NULL) )
			{
				cerr << "Invalid start time specified "
					 << endl << usage;
				exit(1);
			}
			if( !convertTime( args.getArgData(AT_ENDTIME), endTime ) )
			{
				cerr << "Invalid end time specified "
					 << endl << usage;
					exit(1);
			}
			else if( endTime > time(NULL) ) endTime = time(NULL);
				
			if( startTime > endTime )
			{
				cerr << "Invalid time range specified" << endl;
				exit(1);
			}
		}
		else if( args.isset(AT_STARTTIME) && !args.isset(AT_ENDTIME) )
		{
			/* Start time only was specified */
			bTime=true;
			endTime = time(NULL);
			if(!convertTime( args.getArgData(AT_STARTTIME), startTime) ||
					startTime > time(NULL) )
			{
				cerr << "Invalid start time specified "
					 << endl << usage;
				exit(1);
			}
			else if( startTime >= endTime )
			{
				cerr << "Invalid time range specified" << endl;
				exit(1);
			}
		}
		else if( args.isset(AT_ENDTIME) )
		{
			/* End time only was specified */
			bTime=true;
			if( !convertTime( args.getArgData(AT_ENDTIME), endTime ) ||
				endTime > time(NULL) )
			{
				cerr << "Invalid end time specified "
					 << endl << usage;
				exit(1);
			}
		}
		else if( args.isset(AT_ALLDATA) )
		{
			bTime=true;
			endTime = time(NULL);
		}
		else if( args.isset(AT_TAIL) )
		{
			// handling for tailing the log not shown here
		}
		
		// Actual log record store interaction implementation not shown here
	}
	catch(string& s)
	{
		cerr << s << endl;
		cerr << usage << endl;
		exit(1);
	}

	return nRetCode;
}


